#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chalk_1 = require("chalk");
var fs_1 = require("fs");
var jest = require("jest");
var nodemon = require("nodemon");
var path_1 = require("path");
var ramda_1 = require("ramda");
var webpack = require("webpack");
var webpackDevServer = require("webpack-dev-server");
var config_1 = require("./config");
var _a = process.argv, args = _a.slice(2);
var log = console.log;
var rootPath = path_1.resolve(process.cwd());
var requireFx = typeof __webpack_require__ === "function" ? __non_webpack_require__ : require;
var userConfigPath = path_1.join(rootPath, "./typepack.ts");
var userConfig = {};
if (fs_1.existsSync(userConfigPath)) {
    userConfig = requireFx(userConfigPath);
}
var Commands = {
    dev: "dev",
    build_dev: "build:dev",
    build_analyze: "build:analyze",
    build_smp: "build:smp",
    build: "build",
    test: "test"
};
var isDebugging = function () { return !!args.find(function (arg) { return arg === "-d"; }); };
var startWebpack = function (env) {
    var config = config_1.default(env, userConfig, isDebugging());
    config.mode = env;
    webpack(config, function (error, stats) {
        if (error) {
            console.error(error);
            return;
        }
    });
};
var isBackEndConfig = function (config) {
    return config.mode && (config.mode === "server" || config.mode === "cli");
};
switch (args[0]) {
    case Commands.dev:
        var config = config_1.default("development", userConfig, isDebugging());
        config.mode = "development";
        if (isBackEndConfig(userConfig)) {
            webpack(config).watch({}, ramda_1.once(function (error, stats) {
                if (error) {
                    console.error(error);
                    return;
                }
                nodemon({
                    script: path_1.join(rootPath, "./dist/index.js"),
                    watch: path_1.join(rootPath, "./dist/**/*.js")
                }).on("quit", process.exit);
            }));
        }
        else {
            log(chalk_1.default.green("WebpackDevServer Launched"));
            var options = {
                contentBase: path_1.join(rootPath, "./dist"),
                hot: true,
                inline: true,
                stats: { colors: true },
                host: "localhost",
                historyApiFallback: true
            };
            var server = new webpackDevServer(webpack(config), options);
            server.listen(8080, function (error) {
                if (error) {
                    log(chalk_1.default.red(error));
                }
                log(chalk_1.default.green("WebpackDevServer listening at http://localhost:8080"));
            });
        }
        break;
    case Commands.build:
        startWebpack("production");
        break;
    case Commands.build_dev:
        startWebpack("development");
        break;
    case Commands.build_analyze:
        process.env.BUNDLE_ANALYZE = "true";
        startWebpack("production");
        break;
    case Commands.build_smp:
        process.env.SMP = "true";
        startWebpack("production");
        break;
    case Commands.test:
        var conf = {
            projects: [rootPath],
            moduleFileExtensions: ["js", "jsx", "json", "vue", "ts", "tsx"],
            transform: {
                ".+\\.(css|styl|less|sass|scss|png|jpg|ttf|woff|woff2)$": "jest-transform-stub",
                "^.+\\.tsx?$": "ts-jest"
            },
            globals: {
                "ts-jest": {
                    tsConfigFile: path_1.join(rootPath, "tsconfig.json")
                }
            },
            moduleNameMapper: {
                "^@/(.*)$": "<rootDir>/src/$1"
            },
            moduleDirectories: ["node_modules", "src", "tests"],
            testMatch: [
                "**/tests/unit/**/*.spec.(js|jsx|ts|tsx)|**/__tests__/*.(js|jsx|ts|tsx)"
            ],
            testURL: "http://localhost/"
        };
        jest.runCLI(conf, conf.projects);
        break;
    default:
        log(chalk_1.default.red("Unknown Command : " + args));
        break;
}
//# sourceMappingURL=index.js.map