import * as CleanWebpackPlugin from "clean-webpack-plugin";
import * as CopyWebpackPlugin from "copy-webpack-plugin";
import * as FriendlyErrorsWebpackPlugin from "friendly-errors-webpack-plugin";
import * as HtmlWebpackPlugin from "html-webpack-plugin";
import * as MiniCssExtractPlugin from "mini-css-extract-plugin";
import * as OfflinePlugin from "offline-plugin";
import * as OpenBrowserPlugin from "open-browser-webpack-plugin";
import * as OptimizeCssAssetsPlugin from "optimize-css-assets-webpack-plugin";
import { join, resolve } from "path";
import * as SpeedMeasurePlugin from "speed-measure-webpack-plugin";
import * as UglifyJsPlugin from "uglifyjs-webpack-plugin";
import * as webpack from "webpack";
import { BundleAnalyzerPlugin } from "webpack-bundle-analyzer";
import * as webpackMerge from "webpack-merge";
import * as nodeExternals from "webpack-node-externals";
import * as WebpackBar from "webpackbar";

const rootPath = resolve(process.cwd());
const context = rootPath;

const loader = name => require.resolve(name);
const makeDevEntry = (entry, port) => {
  return [
    require.resolve("webpack-dev-server/client") + `?http://localhost:${port}`,
    require.resolve("webpack/hot/dev-server"),
    entry
  ];
};
const resolveLocal = (...args) => {
  return join(__dirname, "../", ...args);
};
const resolveContext = path => {
  return resolve(context, path);
};

const isBackEndConfig = config =>
  config.mode && (config.mode === "server" || config.mode === "cli");

const isCliConfig = config => config.mode && config.mode === "cli";

const configuration = (
  userConfig,
  env = "development",
  isDebugging = false
): webpack.Configuration => {
  const isDev = env !== "production";
  const port = userConfig.port || 8080;

  const paths = {
    src: join(rootPath, "./src"),
    test: join(rootPath, "./test"),
    entry: {
      main: join(rootPath, "./src/index")
    },
    htmlTemplate: join(rootPath, "./src/index.html"),
    outputFolder: join(rootPath, "./dist"),
    ...userConfig.paths
  };

  const getPresets = () => ({
    rules: {
      typescript: {
        test: /\.tsx?$/,
        use: [
          {
            loader: loader("ts-loader"),
            options: { context: rootPath }
          }
        ],
        include: paths.src,
        exclude: /node_modules/
      },
      babel: {
        test: /\.jsx?$/,
        loader: loader("babel-loader"),
        include: paths.src,
        exclude: /node_modules/
      },
      html: {
        test: /\.(html)$/,
        loader: loader("html-loader"),
        include: paths.src,
        exclude: /node_modules/
      },
      scss: {
        test: /\.(sa|sc|c)ss$/,
        use: [
          isDev ? loader("style-loader") : MiniCssExtractPlugin.loader,
          {
            loader: loader("css-loader"),
            options: { sourceMap: !isDev, importLoaders: 1 }
          },
          {
            loader: loader("postcss-loader"),
            options: { sourceMap: !isDev }
          },
          {
            loader: loader("sass-loader"),
            options: { sourceMap: !isDev }
          }
        ]
      },
      images: {
        test: /\.(png|svg|jpg|gif)$/,
        loader: loader("url-loader"),
        options: { limit: 10000, name: "images/[name].[hash:7].[ext]" }
      },
      fonts: {
        test: /\.(woff|woff2|eot|ttf|otf)$/,
        loader: loader("url-loader"),
        options: { limit: 10000, name: "fonts/[name].[hash:7].[ext]" }
      },
      media: {
        test: /\.(mp4|webm|ogg|mp3|wav|flac|aac)(\?.*)?$/,
        loader: loader("url-loader"),
        options: { limit: 10000, name: "media/[name].[hash:7].[ext]" }
      }
    },
    plugins: {
      openBrowser: new OpenBrowserPlugin({ url: `http://localhost:${port}` }),
      webpackbar: new WebpackBar(),
      offline: new OfflinePlugin(),
      friendlyErrors: new FriendlyErrorsWebpackPlugin(),
      miniCssExtract: new MiniCssExtractPlugin({
        filename: "css/[name].[contenthash].css",
        chunkFilename: "css/[id].[chunkhash].css"
      }),
      html: new HtmlWebpackPlugin({
        filename: "index.html",
        template: paths.htmlTemplate,
        inject: true,
        minify: isDev
          ? false
          : {
              removeComments: true,
              collapseWhitespace: true,
              removeAttributeQuotes: true
            }
      }),
      clean: new CleanWebpackPlugin(["dist"], {
        root: rootPath
      }),
      hmr: new webpack.HotModuleReplacementPlugin(),
      analyze: new BundleAnalyzerPlugin(),
      copyStatic: new CopyWebpackPlugin([
        {
          from: join(rootPath, "static"),
          to: paths.outputFolder,
          ignore: [".*"]
        }
      ]),
      banner: new webpack.BannerPlugin({
        banner: "#!/usr/bin/env node",
        raw: true
      })
    }
  });

  const getBaseConfig = () => {
    const { rules, plugins } = mergedConfig;

    return {
      context,
      mode: isDev ? "development" : "production",
      devtool: isDev ? "inline-source-map" : "source-map",
      resolve: {
        extensions: [".tsx", ".ts", ".js", ".jsx", ".json"],
        modules: [
          "node_modules",
          resolveContext("node_modules"),
          resolveLocal("node_modules")
        ]
      },
      resolveLoader: {
        modules: [
          "node_modules",
          resolveContext("node_modules"),
          resolveLocal("node_modules")
        ]
      },
      module: {
        rules: [rules.typescript, rules.babel]
      },
      plugins: [
        plugins.friendlyErrors,
        plugins.webpackbar,
        plugins.clean,
        ...(isDev ? [plugins.hmr] : []),
        ...(process.env.BUNDLE_ANALYZE ? [plugins.analyze] : []),
        ...(process.env.OFFLINE_PLUGIN ? [plugins.offline] : [])
      ],
      optimization: isDev
        ? {}
        : {
            minimizer: [
              new UglifyJsPlugin({
                cache: true,
                parallel: true,
                sourceMap: true
              })
            ]
          }
    };
  };

  const getBackEndConfig = () => {
    return webpackMerge(getBaseConfig(), {
      target: "node",
      externals: [nodeExternals()],
      entry: mergedConfig.paths.entry,
      output: {
        filename: "index.js",
        path: join(rootPath, "./dist"),
        libraryTarget: "commonjs2"
      }
    });
  };

  const getWebAppConfig = () => {
    const { rules, plugins } = mergedConfig;

    return webpackMerge(getBaseConfig(), {
      entry: isDev
        ? { main: makeDevEntry(paths.entry.main, port) }
        : paths.entry,
      module: {
        rules: [rules.html, rules.scss, rules.images, rules.fonts, rules.media]
      },
      plugins: [
        plugins.html,
        plugins.copyStatic,
        ...(isDev ? [plugins.openBrowser] : [plugins.miniCssExtract])
      ],
      output: {
        publicPath: "/",
        filename: isDev ? "js/[name].js" : "js/[name].[chunkhash].js",
        chunkFilename: isDev ? "js/[id].js" : "js/[id].[chunkhash].js",
        path: paths.outputFolder,
        libraryTarget: "umd"
      },
      optimization: isDev
        ? {}
        : {
            minimizer: [
              new OptimizeCssAssetsPlugin({
                cssProcessor: require("cssnano"),
                cssProcessorPluginOptions: {
                  preset: [
                    "default",
                    {
                      discardComments: {
                        removeAll: true
                      }
                    }
                  ]
                },
                canPrint: true
              })
            ]
          }
    });
  };

  const defaultConfig = getPresets();
  const mergedConfig = { ...defaultConfig, ...userConfig.webpack };
  let resultConfig = getWebAppConfig();

  if (isBackEndConfig(mergedConfig)) {
    resultConfig = getBackEndConfig();
  }

  if (isCliConfig(mergedConfig)) {
    resultConfig.plugins.push(mergedConfig.plugins.banner);
  }

  if (isDebugging) {
    console.log(JSON.stringify(resultConfig));
  }

  const smp = new SpeedMeasurePlugin({
    disable: !process.env.SMP
  });
  return smp.wrap(resultConfig);
};

export default configuration;
